package com.cmeeting.service.impl;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.mapper.primary.MeetingInfoMapper;
import com.cmeeting.pojo.MeetingInfo;
import com.cmeeting.service.MeetingInfoService;
import com.cmeeting.util.MinioUtils;
import com.cmeeting.vo.MeetingInfoVO;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class MeetingInfoServiceImpl extends ServiceImpl<MeetingInfoMapper, MeetingInfo>  implements MeetingInfoService {

    @Resource
    private MeetingInfoMapper mapper;
    @Resource
    private MinioUtils minioUtils;

    @Override
    public IPage<MeetingInfo> getPage(MeetingInfoVO vo) {
        LambdaQueryWrapper<MeetingInfo> queryWrapper = new LambdaQueryWrapper<>();
        IPage<MeetingInfo> resultPage = mapper.selectPage(new Page<>(vo.getCurrent(), vo.getSize()), queryWrapper);
        return resultPage;
    }

    @Override
    public boolean updateRecordXml(MeetingInfoVO vo) {
        //前端的表单json转xml，xml格式的纪要内容便于生成会议纪要文件
        String recordJson = vo.getRecordJson();
        String recordXml = convertJSONToXml(recordJson);
        String key = vo.getMeetingId() + "-recordXmlPath-" + IdUtil.fastSimpleUUID() + ".xml";
        minioUtils.upload(key,recordXml.getBytes(StandardCharsets.UTF_8));
        vo.setRecordXml(key);

        LambdaUpdateWrapper<MeetingInfo> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(MeetingInfo::getId,vo.getId());
        updateWrapper.set(MeetingInfo::getUpdateTime,LocalDateTime.now());
        updateWrapper.set(MeetingInfo::getRecordXml,key);
        return update(null,updateWrapper);
    }

    private String convertJSONToXml(String json) {
        JSONArray jsonArray = JSON.parseArray(json);;
        StringBuilder xmlBuilder = new StringBuilder();

        xmlBuilder.append("<root>");
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObj = jsonArray.getJSONObject(i);

            String key = jsonObj.getString("key");
            String label = jsonObj.getString("keyName");
            String value = jsonObj.getString("value");

            xmlBuilder.append(String.format("<%s label=\"%s\">%s</%s>",
                    key, label, value, key));
        }
        xmlBuilder.append("</root>");

        return xmlBuilder.toString();
    }
}
